@@ -1,6 +1,6 @@
 ## function to interactively expand and contract subtrees on a phylogeny
 ## inspired by the phylogeny interface of sharksrays.org by Gavin Naylor
-## written by Liam J. Revell 2015, 2016, 2017
+## written by Liam J. Revell 2015, 2016, 2017, 2018
 
 collapseTree<-function(tree,...){
 	if(!inherits(tree,"phylo")) stop("tree should be an object of class \"phylo\".")
@@ -9,12 +9,13 @@ collapseTree<-function(tree,...){
 	if(hasArg(hold)) hold<-list(...)$hold
 	else hold<-TRUE
 	if(hasArg(drop.extinct)) drop.extinct<-list(...)$drop.extinct
-	else drop.extinct=FALSE
+	else drop.extinct<-TRUE
 	if(is.null(tree$edge.length)){
 		no.edge<-TRUE
 		tree<-compute.brlen(tree,power=0.5)
 	} else no.edge<-FALSE
 	cat("Click on the nodes that you would like to collapse...\n")
+	flush.console()
 	## turn off locator bell (it's annoying)
 	options(locatorBell=FALSE)
 	## check for node labels
@@ -37,75 +38,54 @@ collapseTree<-function(tree,...){
 	points(x=lastPP$xx[1:tree$Nnode+Ntip(tree)],
 		y=lastPP$yy[1:tree$Nnode+Ntip(tree)],pch=21,
 		col="blue",bg="white",cex=1.2)
-	check<-textbox(x=c(par()$usr[1],par()$usr[1]+
-		0.1*(par()$usr[2]-par()$usr[1])),
-		y=par()$usr[4],c("click to stop"),justify="c")
+	rect(par()$usr[1],par()$usr[4]-3*strheight("W"),par()$usr[2],par()$usr[4],
+		border=0,col=make.transparent("blue",0.2))
+	textbox(x=par()$usr[1:2],y=par()$usr[4],
+		c("Click nodes to collapse or expand\nRIGHT CLICK to stop"),
+		justify="c",border=0)		
 	dev.flush()
 	x<-unlist(locator(1))
-	y<-x[2]
-	x<-x[1]
-	d<-sqrt((x-lastPP$xx)^2+(y-lastPP$yy)^2)
-	nn<-which(d==min(d,na.rm=TRUE))
-	## collapse tree & replot:
-	while(!(x>par()$usr[1]&&x<par()$usr[1]+0.1*(par()$usr[2]-par()$usr[1])&&
-		y>check&&y<par()$usr[4])){
-		obj<-list(tree)
-		if(nn>(Ntip(tree)+1)){
-			obj<-splitTree(tree,list(node=nn,
-				bp=tree$edge.length[which(tree$edge[,2]==nn)]))
-			obj[[1]]$tip.label[which(obj[[1]]$tip.label=="NA")]<-
-				tree$node.label[nn-Ntip(tree)]
-			tips<-which(tree$tip.label%in%obj[[1]]$tip.label)
-			theta<-atan(lastPP$yy[nn]/lastPP$xx[nn])
-			if(lastPP$yy[nn]>0&&lastPP$xx[nn]<0) theta<-pi+theta
-			else if(lastPP$yy[nn]<0&&lastPP$xx[nn]<0) theta<-pi+theta
-			else if(lastPP$yy[nn]<0&&lastPP$xx[nn]>0) theta<-2*pi+theta
-			ii<-which((c(tips,Ntip(tree)+1)-c(0,tips))>1)
-			if(ii>1&&ii<=length(tips))
-				tips<-c(tips[1:(ii-1)],theta/(2*pi)*Ntip(tree),tips[ii:length(tips)])
-			else if(ii==1) tips<-c(theta/(2*pi)*Ntip(tree),tips)
-			else if(ii>length(tips)) tips<-c(tips,theta/(2*pi)*Ntip(tree))
-			tree<-read.tree(text=write.tree(obj[[1]]))
-			M<-matrix(NA,min(c(max(4,Ntip(obj[[2]])),10)),length(tips))
-			for(i in 1:ncol(M)) M[,i]<-seq(from=tips[i],to=i,length.out=nrow(M))
-			colnames(M)<-tree$tip.label
-			maxY<-seq(from=sum(sapply(obj,Ntip))-length(obj)+1,to=Ntip(tree),length.out=nrow(M))
-			pw<-reorder(tree,"pruningwise")
-			H<-nodeHeights(tree)
-			for(i in 1:nrow(M)){
-				if(hold) dev.hold()
-				fan(tree,pw,H,xlim=lastPP$x.lim,ylim=lastPP$y.lim,
-					tips=M[i,],maxY=maxY[i],...)
-				if(nodes||i==nrow(M)){
-					lastPP<-get("last_plot.phylo",envir=.PlotPhyloEnv)
-					points(x=lastPP$xx[1:Ntip(tree)],y=lastPP$yy[1:Ntip(tree)],
-						pch=21,col="blue",bg="white",cex=0.8)
-					points(x=lastPP$xx[1:tree$Nnode+Ntip(tree)],
-						y=lastPP$yy[1:tree$Nnode+Ntip(tree)],pch=21,
-						col="blue",bg="white",cex=1.2)
-				}
-				dev.flush()
-			}
-		} else if(nn<=Ntip(tree)) {
-			if(tree$tip.label[nn]%in%otree$node.label){
-				on<-which(otree$node.label==tree$tip.label[nn])+Ntip(otree)
-				obj<-splitTree(otree,list(node=on,
-					bp=otree$edge.length[which(otree$edge[,2]==on)]))
-				nlabel<-tree$tip.label[nn]
-				tree$tip.label[nn]<-"NA"
-				if(nn==1) tips<-c(rep(nn,Ntip(obj[[2]])),(nn+1):Ntip(tree))
-				else if(nn>1&&nn<Ntip(tree)) tips<-c(1:(nn-1),rep(nn,Ntip(obj[[2]])),(nn+1):Ntip(tree))
-				else if(nn==Ntip(tree)) tips<-c(1:(nn-1),rep(nn,Ntip(obj[[2]])))
-				tree<-read.tree(text=write.tree(paste.tree(tree,obj[[2]])))
+	if(!is.null(x)){
+		y<-x[2]
+		x<-x[1]
+		d<-sqrt((x-lastPP$xx)^2+(y-lastPP$yy)^2)
+		nn<-which(d==min(d,na.rm=TRUE))
+		## collapse tree & replot:
+		while(!is.null(x)){
+			obj<-list(tree)
+			if(nn>(Ntip(tree)+1)){
+				obj<-splitTree(tree,list(node=nn,
+					bp=tree$edge.length[which(tree$edge[,2]==nn)]))
+				obj[[1]]$tip.label[which(obj[[1]]$tip.label=="NA")]<-
+					tree$node.label[nn-Ntip(tree)]
+				tips<-which(tree$tip.label%in%obj[[1]]$tip.label)
+				theta<-atan(lastPP$yy[nn]/lastPP$xx[nn])
+				if(lastPP$yy[nn]>0&&lastPP$xx[nn]<0) theta<-pi+theta
+				else if(lastPP$yy[nn]<0&&lastPP$xx[nn]<0) theta<-pi+theta
+				else if(lastPP$yy[nn]<0&&lastPP$xx[nn]>0) theta<-2*pi+theta
+				ii<-which((c(tips,Ntip(tree)+1)-c(0,tips))>1)
+				if(ii>1&&ii<=length(tips))
+					tips<-c(tips[1:(ii-1)],theta/(2*pi)*Ntip(tree),tips[ii:length(tips)])
+				else if(ii==1) tips<-c(theta/(2*pi)*Ntip(tree),tips)
+				else if(ii>length(tips)) tips<-c(tips,theta/(2*pi)*Ntip(tree))
+				tree<-read.tree(text=write.tree(obj[[1]]))
 				M<-matrix(NA,min(c(max(4,Ntip(obj[[2]])),10)),length(tips))
 				for(i in 1:ncol(M)) M[,i]<-seq(from=tips[i],to=i,length.out=nrow(M))
 				colnames(M)<-tree$tip.label
+				maxY<-seq(from=sum(sapply(obj,Ntip))-length(obj)+1,to=Ntip(tree),
+					length.out=nrow(M))
 				pw<-reorder(tree,"pruningwise")
 				H<-nodeHeights(tree)
 				for(i in 1:nrow(M)){
 					if(hold) dev.hold()
 					fan(tree,pw,H,xlim=lastPP$x.lim,ylim=lastPP$y.lim,
-						tips=M[i,],maxY=NULL,...)
+						tips=M[i,],maxY=maxY[i],...)
+					rect(par()$usr[1],par()$usr[4]-3*strheight("W"),
+						par()$usr[2],par()$usr[4],
+						border=0,col=make.transparent("blue",0.2))
+					textbox(x=par()$usr[1:2],y=par()$usr[4],
+						c("Click nodes to collapse or expand\nRIGHT CLICK to stop"),
+						justify="c",border=0)
 					if(nodes||i==nrow(M)){
 						lastPP<-get("last_plot.phylo",envir=.PlotPhyloEnv)
 						points(x=lastPP$xx[1:Ntip(tree)],y=lastPP$yy[1:Ntip(tree)],
@@ -116,20 +96,57 @@ collapseTree<-function(tree,...){
 					}
 					dev.flush()
 				}
+			} else if(nn<=Ntip(tree)) {
+				if(tree$tip.label[nn]%in%otree$node.label){
+					on<-which(otree$node.label==tree$tip.label[nn])+Ntip(otree)
+					obj<-splitTree(otree,list(node=on,
+						bp=otree$edge.length[which(otree$edge[,2]==on)]))
+						nlabel<-tree$tip.label[nn]
+					tree$tip.label[nn]<-"NA"
+					if(nn==1) tips<-c(rep(nn,Ntip(obj[[2]])),(nn+1):Ntip(tree))
+					else if(nn>1&&nn<Ntip(tree)) 
+						tips<-c(1:(nn-1),rep(nn,Ntip(obj[[2]])),(nn+1):Ntip(tree))
+					else if(nn==Ntip(tree)) tips<-c(1:(nn-1),rep(nn,Ntip(obj[[2]])))
+					tree<-read.tree(text=write.tree(paste.tree(tree,obj[[2]])))
+					M<-matrix(NA,min(c(max(4,Ntip(obj[[2]])),10)),length(tips))
+					for(i in 1:ncol(M)) M[,i]<-seq(from=tips[i],to=i,length.out=nrow(M))
+					colnames(M)<-tree$tip.label
+					pw<-reorder(tree,"pruningwise")
+					H<-nodeHeights(tree)
+					for(i in 1:nrow(M)){
+						if(hold) dev.hold()
+						fan(tree,pw,H,xlim=lastPP$x.lim,ylim=lastPP$y.lim,
+							tips=M[i,],maxY=NULL,...)
+						rect(par()$usr[1],par()$usr[4]-3*strheight("W"),
+							par()$usr[2],par()$usr[4],
+							border=0,col=make.transparent("blue",0.2))
+						textbox(x=par()$usr[1:2],y=par()$usr[4],
+							c("Click nodes to collapse or expand \nRIGHT CLICK to stop"),
+							justify="c",border=0)
+						if(nodes||i==nrow(M)){
+							lastPP<-get("last_plot.phylo",envir=.PlotPhyloEnv)
+							points(x=lastPP$xx[1:Ntip(tree)],y=lastPP$yy[1:Ntip(tree)],
+								pch=21,col="blue",bg="white",cex=0.8)
+							points(x=lastPP$xx[1:tree$Nnode+Ntip(tree)],
+								y=lastPP$yy[1:tree$Nnode+Ntip(tree)],pch=21,
+								col="blue",bg="white",cex=1.2)
+						}
+						dev.flush()
+					}
+				}
+			} else { 
+				cat("Collapsing to the root is not permitted. Choose another node.\n")
+				flush.console()
+			}
+			x<-unlist(locator(1))
+			if(!is.null(x)){
+				y<-x[2]
+				x<-x[1]
+				lastPP<-get("last_plot.phylo",envir=.PlotPhyloEnv)
+				d<-sqrt((x-lastPP$xx)^2+(y-lastPP$yy)^2)
+				nn<-which(d==min(d,na.rm=TRUE))
 			}
-		} else { 
-			cat("Collapsing to the root is not permitted. Choose another node.\n")
-			flush.console()
 		}
-		check<-textbox(x=c(par()$usr[1],par()$usr[1]+
-			0.1*(par()$usr[2]-par()$usr[1])),y=par()$usr[4],
-			c("click to stop"),justify="c")
-		x<-unlist(locator(1))
-		y<-x[2]
-		x<-x[1]
-		lastPP<-get("last_plot.phylo",envir=.PlotPhyloEnv)
-		d<-sqrt((x-lastPP$xx)^2+(y-lastPP$yy)^2)
-		nn<-which(d==min(d,na.rm=TRUE))
 	}
 	## turn locator bell back on
 	options(locatorBell=TRUE)
